// main.cpp

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_opengl.h>

#include "Utility.h"
#include "Player.h"

//======================================================================
// Constants
//======================================================================

const int SCR_FULLSCREEN = false;
const int MAX_FPS = 25;
const int NUM_PLAYERS = 2;

//======================================================================
// Prototypes
//======================================================================

// Library init and deinit functions
void Init(void);
void InitGL(void);
void Quit(void);

// Data functions
void LoadData(void);
void UnloadData(void);

// Main loop stuff
void MainLoop(void);
void Input(void);
void Update(void);
void Draw(void);

//======================================================================
// Variables
//======================================================================

SDL_Surface *screen;
bool done;

GLuint backgroundTex, bulletTex;

Player *players[NUM_PLAYERS];

//======================================================================
// Implementation
//======================================================================

int main(int argc, char **argv)
{
	Init();
	LoadData();

	InitGL();

	MainLoop();

	UnloadData();
	Quit();
	return 0;
}

void Init(void)
{
	int result;
	Uint32 scrFlags = SDL_OPENGL;
	int colorSize;

	result = SDL_Init(SDL_INIT_VIDEO);
	if(result == -1)
		Error("Can't initialize Simple DirectMedia Layer!");

	if(SCR_BPP == 16)
		colorSize = 5;
	else if(SCR_BPP == 32)
		colorSize = 8;
	else
		Error("Wrong bits per pixel for screen!");

	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, colorSize);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, colorSize);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, colorSize);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	if(SCR_FULLSCREEN)
		scrFlags += SDL_FULLSCREEN;

	screen = SDL_SetVideoMode(SCR_W, SCR_H, 0, scrFlags);
	if(!screen)
		Error("Can't set video mode!");

	SDL_WM_SetCaption("glDuel", NULL);
}

void InitGL(void)
{
	glViewport(0, 0, SCR_W, SCR_H);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0, SCR_W, SCR_H, 0, -1.0f, 1.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void Quit(void)
{
	SDL_Quit();
}

void LoadData(void)
{
	int i;

	backgroundTex = LoadTexture("images/background.jpg");
	bulletTex = LoadTexture("images/bullet.bmp", true);

	for(i=0; i<NUM_PLAYERS; i++)
		players[i] = new Player(i + 1);
}

void UnloadData(void)
{
	int i;

	FreeTexture(&backgroundTex);
	FreeTexture(&bulletTex);

	for(i=0; i<NUM_PLAYERS; i++)
		delete players[i];
}

void MainLoop(void)
{
	Uint32 sTime;
	done = false;

	do
	{
		sTime = SDL_GetTicks();

		Input();
		Update();
		Draw();

		SDL_GL_SwapBuffers();

		while(SDL_GetTicks() - sTime < 1000 / MAX_FPS);

	} while(!done);
}

void Input(void)
{
	int i;
	static SDL_Event event;
	static Uint8 *keys = SDL_GetKeyState(NULL);

	while(SDL_PollEvent(&event))
		if(event.type == SDL_QUIT)
			done = true;

	SDL_PumpEvents();

	if(keys[SDLK_ESCAPE])
		done = true;

	for(i=0; i<NUM_PLAYERS; i++)
		players[i]->Input(keys);
}

void Update(void)
{
	int i, j;
	
	for(i=0; i<NUM_PLAYERS; i++)
	{
		players[i]->Update();
		
		for(j=0; j<NUM_PLAYERS; j++)
		{
			if(j == i)
				continue;
				
			if(players[i]->BulletCollWith(players[j]))
				players[j]->Hit();
		}
	}
}

void Draw(void)
{
	int i;

	/*
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	*/

	glEnable(GL_TEXTURE_2D);
	DrawTexture(backgroundTex, 0, 0, SCR_W, SCR_H);

	for(i=0; i<NUM_PLAYERS; i++)
		players[i]->Draw();
}
